//-----------------------------------------------------------------------------
//	File: opfile.cpp
//  Description: Far Editor Plugin for Open Files on press Ctrl+Enter
//  Copyright (C)2002++ by Volodimir V. Flyonts
//-----------------------------------------------------------------------------
// Last Edit: 19:25 November 27, 2002

#include <windows.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include "plugin.hpp"

//-----------------------------------------------------------------------------
//-  ⮣ ᥣ  稭 :)
static _export char copyright[]="Copyright (C)2002++ by Volodimir V. Flyonts";

typedef struct _FOUND {
	unsigned int path_index;
	const char * file_name;
    const char * file_path;
} FOUND, _FND, *LPFND;

typedef	struct FarMenuItem FMI, *LPFMI;

//-----------------------------------------------------------------------------

#define	PWL			0x200	// #include string
#define	PWC			0x100	// - pathways
#define	WFC			0x100	// - wasfound

//-----------------------------------------------------------------------------

static HANDLE hModule;
static struct PluginStartupInfo		Info;
static struct FarStandardFunctions	FSF;
static char	s_pathways[ PWL ];		//  ப ...  8212
static char * pathways[ PWC ];		//  ᪠
static _FND	 was_found[ WFC ];		//  䠩
static char  find_mask[ MAX_PATH ]; // ᪮ 䠩 (᪠)
static char *find_file_name;		// 㪠⥫  
static char *find_file_path;        // 㪠⥫  
static unsigned int  pw_count;      // ᪮쪮 ⥩
static unsigned int  wf_count;      // ᪮쪮  䠩
static int active;	//   祣 ࠡ  㤥 :)

//-----------------------------------------------------------------------------
static struct _REG_STR {
	const char *sss;
	const char *pathways;
	const char *active;
} reg_key= {
	"%s%s%s",
	"PathWays",
	"Active"
};
//-----------------------------------------------------------------------------

enum {
	STitle,
    SCfgTitle,
	SActive,
	SPathWays,
	SSave,
	SCancel,
	SNone
};//enum

//-----------------------------------------------------------------------------
#include "reg.cpp"
#include "mem.cpp"
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
/* DllMain */
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
inline const char *get_msg(int id){return(Info.GetMsg(Info.ModuleNumber,id));};
//-----------------------------------------------------------------------------

static char def_path[ MAX_PATH ];
const char * get_def_path(void)
{
	static struct EditorInfo EInfo;
	memset(def_path, 0, MAX_PATH);
	Info.EditorControl(ECTL_GETINFO, &EInfo);
	char *c, *p = (char *)EInfo.FileName;
	while((c=strchr(p, '\\'))!=NULL)
		p = c + 1;
	if(p==EInfo.FileName){
		if(GetCurrentDirectory(MAX_PATH-1, def_path)==0)
			return(NULL);
		return(def_path);
	};//if
	strncpy(def_path, EInfo.FileName, p-1-EInfo.FileName);
	return(const char*)(def_path);
};//get_def_path

//-----------------------------------------------------------------------------

int build_pathways(void)
{
	pw_count	= 0;
	pathways[0]	= (char*)get_def_path();
	if(!pathways[0])
		return(pw_count);
	pw_count	= 1;
/*
	if(f_path)
		delete[] f_path;
	unsigned long f_size;
*/
	int e_code;
/*
    //- GetRegKeySize -----------------------------------------------
	e_code	= GetRegKeyEx(HROOT, "", reg_key.pathways, NULL, NULL, &f_size);
	if(!((e_code)&&(f_size)))
		return(pw_count);
	f_path = new char[f_size+1];
	if(!f_path)
		return(pw_count);
*/
    //- GetString ---------------------------------------------------
	e_code	= GetRegKey(HROOT, "", reg_key.pathways, s_pathways, "", PWL);
	if(!((e_code)&&(*s_pathways)))
		return(pw_count);
	char *c, *p = s_pathways;
	while((c=strchr(p, ';'))!=NULL){
		pathways[pw_count] = p;
		*c	= 0;
		p	= c+1;
		pw_count++;
	};//while
	pathways[pw_count] = p;
	return(++pw_count);
};//build_pathways

//-----------------------------------------------------------------------------

inline int file_exists(const char * fname)
{
	return(GetFileAttributes(fname)!=(DWORD)(-1));
};//file_exists

//-----------------------------------------------------------------------------
//  ⮫쪮  ᪨  䠫
inline int is_char(char c)
{
	//  ਢ   (. ᫮)
	return(((c>='A')&&(c<='Z'))||((c>='a')&&(c<='z'))||
			((c>='0')&&(c<='9'))||(c=='\\')||
			(c=='/')||(c=='.')||(c=='-')||(c=='$')||(c=='_')|| 
			(c=='!')||(c=='@')||(c=='#')||(c=='?')||(c=='*')); // 䨫 ⮦
};//is_char

//-----------------------------------------------------------------------------
//      䠩  ஬
const static char path_root[] = "\\";
int get_word(char *fn, const char *str, int pos, int sln)
{
	int i,j;
	//-  砫 ᫮ -----------------------------------------
	for(i=pos;(i>0)&&(is_char(str[i]));i--)
		;
	if(is_char(str[i]))	i--;
	//-   ᫮ ------------------------------------------
	// ࠭ ࠭祭 :) //  ! ( ᫥  ࠭)
	for(j=pos;(j<sln)&&(is_char(str[j]));j++)
		;
	if(is_char(str[j]))	j++;
	//- ᮡ⢥ ᨬ ------------------------------------------
	j -= i+1;
	if(j<=0)	// ᫨    䠩 ...
		return(FALSE);	
	strncpy(fn, str+i+1, j);
    fn[j] = 0;	//  ஥  뢠  
	return(TRUE);
};//get_word


//-----------------------------------------------------------------------------

int get_cursor_pos(int &x, int &y, int in_file = 1)
{
	static struct EditorInfo 		EInfo;  // editor  info
	if(!Info.EditorControl(ECTL_GETINFO, &EInfo))
		return(FALSE);
	if(in_file){
    	x	= EInfo.CurPos;
    	y	= EInfo.CurLine;
    }else{
    	x	= EInfo.CurTabPos;
    	y	= EInfo.CurLine - EInfo.TopScreenLine;
    };//if
    return(TRUE);
};//get_cursor_pos

//-----------------------------------------------------------------------------

/*
** 둫 ... . [*], ⥯  !
** -   㤥 祭 ࠭ ࠡ ᫨  䠩 "\file.ext"
**       ୥ ⠫ ...
*/
/*  襭ᠭ ५    ய饭 :) */

int get_filename()
{
	static struct EditorGetString	EStr;	// editor  string
	EStr.StringNumber	= -1;				// current string
	if(!Info.EditorControl(ECTL_GETSTRING, &EStr))
		return(FALSE);
    int mx, my;
    if(!get_cursor_pos(mx, my))
    	return(FALSE);
	if(!get_word(find_mask, EStr.StringText, mx, strlen(EStr.StringText)))
		return(FALSE);
	//-  ⤥쭮  ⤥쭮 ----------------------------------
	char *beg, *end;
    int len = strlen(find_mask);
	for(int i=0;i<len;i++)
		if(find_mask[i]=='/')
			find_mask[i]='\\';
	beg	= find_mask;
	while((end=strchr(beg, '\\'))!=NULL)
		beg	= end + 1;
	//- ᫨       -
	if(beg==find_mask){
		find_file_name	= find_mask;
		find_file_path	= NULL;
		return(TRUE);
	};
	//-  --------------------------
	if((beg-find_mask==1)&&(find_mask[0]=='\\')){
	// [*] // , ஥ ᫮  譥	
		find_file_name = beg;
		find_file_path = (char *)path_root;
		return(TRUE);
	};//if
	*(beg-1) = 0;
	find_file_name = beg;
	find_file_path = find_mask;
	return(TRUE);
};//get_filename

//-----------------------------------------------------------------------------
//   饬 䠩, ᫨   롨ࠥ 㦭   ...
static char last_path[ MAX_PATH ]; //   lost ...
#ifdef _DEBUG
static char temp_path[ MAX_PATH ]; // ⮫쪮  ⫠
#endif
int find_file()
{
	GetCurrentDirectory(MAX_PATH-1, last_path);
	wf_count = 0;
	//---------------------
	HANDLE			HF;
	WIN32_FIND_DATA	FD;
	//---------------------
for(int m2=0;m2<2;m2++){ //  室  筮 ᮮ. ன - any ext.
	for(unsigned int i=0;i<pw_count;i++){
		if(!SetCurrentDirectory(pathways[i]))
			continue;
		if(find_file_path)
		if(!SetCurrentDirectory(find_file_path))
			continue;
#ifdef _DEBUG
        GetCurrentDirectory(MAX_PATH-1, temp_path);
#endif
		if((HF=FindFirstFile(find_file_name, &FD))==INVALID_HANDLE_VALUE)
			continue;
        unsigned int len;
        do{
        	len = strlen(FD.cFileName);
			was_found[wf_count].file_name	= new char[len+1];
            was_found[wf_count].file_path	= NULL;
            was_found[wf_count].path_index	= i;
            strcpy((char*)was_found[wf_count].file_name, FD.cFileName);
            wf_count++;
        }while(FindNextFile(HF, &FD));
		FindClose(HF);
	};//for
    if(wf_count==0) strcat(find_file_name, ".*");else	break;
};//for        

	SetCurrentDirectory(last_path);
    if(wf_count<2)	//  ...
    	return(wf_count-1);
    LPFMI Items = new FMI[wf_count];
    for(unsigned int i=0;i<wf_count;i++){
    	memset(&Items[i], 0, sizeof(FarMenuItem));
        FarSprintf(Items[i].Text, "%-12s in %s",
    			   was_found[i].file_name,
                   pathways[was_found[i].path_index]);
    };//for
    int mx, my;
    get_cursor_pos(mx, my, 0);
    int menu_code = Info.Menu(Info.ModuleNumber,
    						  mx, my+3, 0, //  0  ⮮।
                              FMENU_SHOWAMPERSAND | FMENU_WRAPMODE,
                              NULL, NULL, NULL, NULL, NULL,
                              Items, wf_count);
    delete[] Items;
    return(menu_code);
};//find_file

//-----------------------------------------------------------------------------

void free_find_data()
{
	if(wf_count)	// :)
	for(register unsigned int i=0;i<wf_count;i++){
    	if(was_found[i].file_name)
        	delete[] was_found[i].file_name;
        if(was_found[i].file_path)
        	delete[] was_found[i].file_path;
    };//for
    wf_count = 0;
};//free_find_data

//-----------------------------------------------------------------------------

int get_full_file_name(char * dest, LPFND fnd)
{
    if(fnd->file_path)
    	strcpy(dest, fnd->file_path);
    else
    	strcpy(dest, pathways[fnd->path_index]);
    if(find_file_path){
    	if(dest[strlen(dest)-1]!='\\')
    		strcat(dest, "\\");
    	strcat(dest, find_file_path);
    };//if
    if(dest[strlen(dest)-1]!='\\')
    	strcat(dest, "\\");
    strcat(dest, fnd->file_name);
	return(TRUE);
};//get_full_file_name

//-----------------------------------------------------------------------------
//   FarDialogItem
void fdi(struct FarDialogItem *I, 
		 int t, int x1, int y1, int x2, int y2, 
		 int f, int s, DWORD fl, int db, const char *dt)
{
	I->Type	= t;
	I->X1	= x1;
	I->Y1	= y1;
	I->X2	= x2;
	I->Y2	= y2;
	I->Focus	= f;
	I->Selected	= s;
	I->Flags	= fl;
	I->DefaultButton	= db;
	if(dt)
		strcpy(I->Data, dt);
};//FDI

//-----------------------------------------------------------------------------

#define	EDF	DIF_VAREDIT
#define DBF	DIF_LEFTTEXT		/*DI_DOUBLEBOX */
#define BTF	DIF_CENTERGROUP    /* | DIF_NOBRACKETS*/

void config_plugin()
{                                     
	GetRegKey(HROOT, "", reg_key.pathways, s_pathways, "", PWL);
		                              //     1     2    3     4      5    6   7
	static struct FarDialogItem DI[7];// ࠬ   ⥪ cancel save sep
	fdi(&DI[0], DI_DOUBLEBOX, 3, 1,64, 6,  0, 0, 0, 0, get_msg(SCfgTitle)); 
	fdi(&DI[1], DI_TEXT,      5, 2, 0, 0,  0, 0, 0, 0, get_msg(SPathWays));
    fdi(&DI[2], DI_TEXT,      3, 4, 0, 0,  0, 0, DIF_SEPARATOR, 0, NULL);
	fdi(&DI[3], DI_EDIT,      5, 3,62, 0,  1, 0, 0, 0, s_pathways );
	fdi(&DI[4], DI_CHECKBOX,  5, 5, 0, 0,  0, active, 0, 0, get_msg(SActive));
	fdi(&DI[5], DI_BUTTON,   28, 5, 0, 0,  0, 1, BTF, 1, get_msg(SSave));
	fdi(&DI[6], DI_BUTTON,   42, 5, 0, 0,  0, 0, BTF, 0, get_msg(SCancel));
	int d_code	= Info.Dialog(Info.ModuleNumber,
							  -1, -1, 68, 8,
							  NULL, DI, sizeof(DI)/sizeof(*DI));
	if(d_code!=5) // if not save config
		return;
	SetRegKey(HROOT, "", reg_key.active, (active=DI[4].Selected));
	SetRegKey(HROOT, "", reg_key.pathways, DI[3].Data);
};//config_plugin

//-----------------------------------------------------------------------------

int do_main(void)
{
	//- 㧨  ᪠  ॥ ---------------------
	if(!build_pathways())
	  	return(1);
	//- 稬  䠩  ஬ ----------------------
	if(!get_filename())
		return(1);
	//- 饬, 饬 --------------------------------------
    int pos;
	if((pos = find_file())<0)
		return(1);
	static char open_file_name[ MAX_PATH ];
    if(!get_full_file_name(open_file_name, &was_found[pos]))
		return(1);
	free_find_data();
	if(!file_exists(open_file_name))
		return(1);
	if(Info.Editor(open_file_name,
	   				NULL,
	   				0,0,-1,-1,
	   				EF_NONMODAL|EF_ENABLE_F6,
		0, 1)==EEC_OPEN_ERROR){
		    	return(1);	//    뢥 ᮮ饭  訡
	};//if
};//do

//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
int WINAPI _export GetMinFarVersion(void){ return(MAKEFARVERSION(1,50,0)); };
//-----------------------------------------------------------------------------

void WINAPI _export SetStartupInfo(const struct PluginStartupInfo *PSInfo)
{
	::Info		= *PSInfo;
	FSF			= *PSInfo->FSF;
	Info.FSF	= &FSF;
	FarSprintf(PluginRegRootKey, "%s\\%s", Info.RootKey, "F_Open");
	GetRegKey(HROOT, "", reg_key.active, active, 1);
};

//-----------------------------------------------------------------------------

void WINAPI _export GetPluginInfo(struct PluginInfo *PInfo)
{
	PInfo->StructSize	= sizeof(struct PluginInfo);
	PInfo->Flags		= PF_DISABLEPANELS | PF_EDITOR; // | PF_PRELOAD;
	static char *PMStrings[1];
  	PMStrings[0] = (char *)get_msg(STitle);
  	PInfo->PluginMenuStrings=PMStrings;
  	PInfo->PluginMenuStringsNumber=sizeof(PMStrings)/sizeof(*PMStrings);
  	PInfo->PluginConfigStrings=PMStrings;
  	PInfo->PluginConfigStringsNumber=sizeof(PMStrings)/sizeof(*PMStrings);
};//GetPluginInfo

//-----------------------------------------------------------------------------

HANDLE WINAPI _export OpenPlugin(int, int)
{
//	config_plugin();
	do_main();	// ⥯   ...
	return(INVALID_HANDLE_VALUE);	
};//OpenPlugin

//-----------------------------------------------------------------------------

int WINAPI _export Configure(int)
{
	config_plugin();
	return(FALSE);
};//Configure

//-----------------------------------------------------------------------------

int WINAPI _export ProcessEditorInput(const INPUT_RECORD *Rec)
{
	if(active && Rec->EventType==KEY_EVENT)
		if((Rec->Event.KeyEvent.bKeyDown)&&					//   ᭮
		   (Rec->Event.KeyEvent.wVirtualKeyCode==0x0D)&&	// 13 ⨯ Enter
		  ((Rec->Event.KeyEvent.dwControlKeyState&LEFT_CTRL_PRESSED)||
		   (Rec->Event.KeyEvent.dwControlKeyState&RIGHT_CTRL_PRESSED))){
			/*    㦭, ⥯ ⮯१㧪 ᥣ -----
		    //- ᫨  ⮬ - १㧨  ᪠ ------------
		    if((Rec->Event.KeyEvent.dwControlKeyState&LEFT_ALT_PRESSED)||
		       (Rec->Event.KeyEvent.dwControlKeyState&RIGHT_ALT_PRESSED)){
		    	build_pathways();
		    };//if */
			return(do_main());
		};//if
	return(0);
};//ProcessEditorInput

//-----------------------------------------------------------------------------
//EOF